#!/bin/bash
#####################################################################
#File Name: make_islParFile.sh
#Author:Benjamin Peter
#Modified by: Benjamin Peter
#Creation Date: unknown
#Description: Writes ParFile for the island model
#Last Modification: 2009-10-27 
#####################################################################
#Arguments:
#  1. Number of demes
#  2. Individuals per deme
#  3. Sampling Parameter
#  4. Number of Migrants
#  5. Number of Loci
#  6. Time to converge to 1 pop (0 = no convergence)
#  7. SampleSize
#  8. AncientSize"
	
#####################################################################
function round (){
  echo $(printf %.0f $1)
}
############################################################
#note that the srand() seeding depends on time. Thus, using a bash loop won't work
function getRandomNumber (){ 
     n=$1
     max=$2
     AWKSCRIPT="BEGIN {
                 srand()
               }
               {
                 for(i=1;i<=$n;i++)
                 print int($max*rand())+1; exit
               }"
     echo | awk "$AWKSCRIPT"
}

#####################################################################
totNumDemes=`round $1`;
demeSize=`round $2`;
samplingParameter=$3;
numberOfMigrants=$4;
numLociToSimulate=`round $5`;
timeToConverge=`round $6`;
sampSize=`round $7`;
ancientSize=`round $8`;


mutationRate="%ALPHA%MU"
#Prepare simcoal input file with a hierarchical island genetic structure
#Writing  par file
parFile=DEFAULT.par

#Define basic properties of the data to be generated
echo "sampSize: $sampSize";
#round samplingParameter to integer
samplingParameter=$(echo | awk "{printf( \"%.0f\", $samplingParameter*$sampSize+$samplingParameter-0.49) }")
dataType="MICROSAT";
recRate=0;
geomParameterSTR=0;
rangeConstraint=0;


#Sampling individuals
#version with discrete sampling parameter

#first, initialize Vector
for i in `seq $totNumDemes`; do    
     sampleVector[$i]=0
done
sampleVector[1]=$((2*$samplingParameter));

#get Sampling Demes
a=`getRandomNumber $((sampSize-samplingParameter)) $((totNumDemes-1))`
#a: no in 1..D-1; add 1 to every entry
b=`for i in $a; do echo $((i+1)); done`

#then, add for each entry in b 2 sampled genes to the sample vector
for sampleSite in $b; do
     sampleVector[$sampleSite]=$((sampleVector[$sampleSite]+2));
     echo $sampleSite;    
done
              
#calculate single migration rate
m_single=0;
if [ $totNumDemes -gt 1 ]; then
     script="{print $numberOfMigrants/($demeSize*($totNumDemes-1));}"
     m_single=`echo | awk "$script"`
fi

#Preparing migration matrix
for i in `seq $totNumDemes`; do    
     M_col[$i]=$m_single
done

M=`for i in $(seq $totNumDemes); do
     echo "${M_col[@]}";
done`

for i in `seq $totNumDemes`; do    
     M_col[$i]=0
done

noMigrationMatrix=`for i in $(seq $totNumDemes); do
     echo "${M_col[@]}";
done`

echo "//Parameters for the coalescence simulation program : simcoal.exe">$parFile
echo "$totNumDemes samples to simulate">>$parFile

echo "//Population effective sizes (number of genes = 2*num diploids)">>$parFile

for i in `seq $totNumDemes`; do
  echo $demeSize >>$parFile
done

echo "//Samples sizes (number of genes); (2N = $((2*$sampSize)))" >>$parFile
for i in `seq $totNumDemes`; do
     echo ${sampleVector[$i]} >>$parFile
done

echo "//Growth rates" >>$parFile
for i in `seq $totNumDemes`; do
     echo "0" >> $parFile
done

echo "//Number of migration matrix" >>$parFile
echo "2">>$parFile

echo "//Migration rates matrix  0">>$parFile
echo "$M" >>$parFile

echo "//Migration rates matrix  0">>$parFile
echo "$noMigrationMatrix" >>$parFile

echo "//Historical events" >>$parFile
if [ $timeToConverge -eq 0 ]; then
     echo "0" >> $parFile
else
     echo "$totNumDemes historical events" >> $parFile
     echo "$timeToConverge 0 0 1 `echo | awk "{print $ancientSize/$demeSize;}"` 0 1">>$parFile
     for i in `seq $((totNumDemes-1))`; do
          echo "$timeToConverge $i 0 1 1 0 1" >> $parFile
     done
fi


echo "//Number of independent loci" >> $parFile
echo "$numLociToSimulate 0" >> $parFile
for i in `seq $numLociToSimulate`; do
     echo "//Number of contiguous linkage blocks in chromosome $i" >> $parFile
     echo "1"  >> $parFile
     echo "//Per Block: Data type, No. of loci, Recombination rate to the right-side locus, plus optional parameters" >> $parFile
     echo "$dataType 1 $recRate $mutationRate 0 0"  >> $parFile
done

echo "finished writing parfile";

