######################################################################
#File Name: modelChoice_Pritchard.r
#File Type: R-Script
#Author: Benjamin Peter, after a Script by Laurent Excoffier
#Creation: 27.10.20 09:58:21
#Last Modification: 27.10.20 09:58:21
#Description: implements model choice procedure based on ABC described
#   in Pritchard et. al. (1999)
#Requires: obs-file(with header) with observed Summary stats
#          ABC-output (*bestSims_ParamStats*)-Files with 5k best sims
#          this is a version that does not produce any diagnostic graphs
#output: rankChoice.out with P(Models) for various distance thresholds
######################################################################



#===========================================
EuclDist<-function(SSVec1, SSVec2) {
	dist<-0;
	dist<-sqrt(sum((SSVec1-SSVec2)^2));
	return (dist);
}

#===========================================
normalize<-function(vec) {
	stdev<-sd(vec);
	if (stdev!=0) {
	return ((vec-mean(vec))/stdev);
	}
	else return (vec);
}
#===========================================
getFileFuzzy<-function(s){
	return(list.files()[grep(s,list.files())])
}      
#===========================================
obsFile<-getFileFuzzy(".obsp");
zz <- file("rankChoice.out", "w")  # open an output file connection

#distances to calculate Probabilites for respective models
widthIntervals=1;   
minNumber=1;
maxNumber=10;
xvals<-seq(minNumber, maxNumber, widthIntervals);

#create header

header<-c("ID")
for (x in xvals){
header<-c(header,paste("PBneck_",x,sep=""))
header<-c(header,paste("PIsland_",x,sep=""))
}

cat(header,sep="\t",file=zz)
cat("\tMannWithneyTest")
cat("\n",file=zz)

obsMatrix    <- as.matrix (read.table(obsFile, header=T, sep="\t", na.strings = "NA"));
a<-data.frame()

models<-getFileFuzzy("bestSims.txt")

obs<-as.vector(obsMatrix[1,])
obs<-obs[!(is.na(obs))]
file1<-models[1]
file2<-models[2]
       	
#Read all files and standardize sumstats
model1 <- read.table(file1, header=T, comment.char="");
model2 <- read.table(file2, header=T, comment.char="");

numSS= length(obs)  
   
#Just take the first numSS sum stats
model1<-model1[(length(model1)-numSS+1):length(model1)]
model2<-model2[(length(model2)-numSS+1):length(model2)]

   
#Combine everything into a single big matrix
allData<-rbind(model1, model2);

#pairs(allData[1:100, 1:numSS])
   
allData_s<-allData;
   
obs_s<-rep(0,numSS)
#Standardize each SS and observations
for (i in 1:numSS) {
allData_s[,i]<-normalize(allData[,i]);
obs_s[i]<-(obs[i]-mean(allData[,i]))/sd(allData[,i]);
}
#Compute distances between observations and simulations
totNumSims<-length(allData_s[,1]);
dist<-rep(0,totNumSims);
for (i in 1:totNumSims) {
	dist[i]<-EuclDist(obs_s,allData_s[i,]);
}
   
#plot densities
numSimsM1<-nrow(model1);
numSimsM2<-nrow(model2);
bandwidth=0.09
d1<-density(dist[1:numSimsM1], bw=bandwidth);
d2<-density(dist[(numSimsM1+1):(numSimsM1+numSimsM2)], bw=bandwidth);
maxx<-max(d1$x,d2$x);
maxy<-max(d1$y,d2$y);
   
#Rank the distances
ranks<-rank(dist, ties.method= "min");
ranks_d1<-ranks[1:numSimsM1];
ranks_d2<-ranks[(numSimsM1+1):(numSimsM1+numSimsM2)];
   
#Plot the relative probabilities at given intervals
numCats=(maxNumber-minNumber)/widthIntervals;
m1prob<-rep(0,numCats);
m2prob<-rep(0,numCats);

j<-0;

cat(x,"\t",file=zz)

for (i in xvals) {
	j<-j+1;
	w1<-length(ranks_d1[ranks_d1[]<i+1]);
	w2<-length(ranks_d2[ranks_d2[]<i+1]);

	totw<-w1+w2;
	m1prob[j]<-w1/totw; 
	m2prob[j]<-w2/totw;

	cat(m1prob[j],m2prob[j],"",sep="\t",file=zz)
}               
modelstochosefrom<-c(rep(0,numSimsM1),rep(1,numSimsM2))
m.factor<-as.factor(modelstochosefrom)
pval<-wilcox.test(ranks~m.factor,alternative="less")$p.value
cat(pval,file=zz);
cat(pval);
cat("\n",file=zz);
#find posterior probabilty for a fixed distance threshold
positionToCompute=10;
numBestDistances=positionToCompute;
xvalIndices<-rank(xvals);
curIndex<-xvalIndices[xvals[]==numBestDistances];
postProba=rep(0,2);
postProba[1]<-m1prob[curIndex];
postProba[2]<-m2prob[curIndex];

close(zz)
   
   
   
   
   
   
   
   
   
